/***************************
GTK Pattern Creation Lab
Copyright (C) 1999  Peter Stuart

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************/


#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "types.h"
#include "gpcl.h"
#include "gpcldlgs.h"
#include "gpclpatmenu.h"

extern tpreferences gpreferences;
extern GtkWidget *generate_button;
extern GtkWidget *interrupt_button;
extern GtkWidget *progress_bar;
extern GtkWidget *progress_label;

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *found_widget;

  if (widget->parent)
    widget = gtk_widget_get_toplevel (widget);
  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
/*
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
*/
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

GtkWidget*
create_mainwindow ()
{
  GtkWidget *mainwindow;
  GtkWidget *vbox;
  GtkWidget *configurebox;
  GtkWidget *menubar2;
  GtkWidget *file;
  GtkWidget *menu19;
  GtkWidget *file_new;
  GtkWidget *file_open;
  GtkWidget *file_save;
  GtkWidget *file_save_as;
  GtkWidget *separator1;
  GtkWidget *file_preferences;
  GtkWidget *separator2;
  GtkWidget *file_quit;
  GtkWidget *pattern;
  GtkWidget *menu20;
  GtkWidget *help;
  GtkWidget *menu21;
  GtkWidget *help_about;

  mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect (GTK_OBJECT (mainwindow), "delete_event",
                      GTK_SIGNAL_FUNC (on_mainwindow_end_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (mainwindow), "destroy_event",
                      GTK_SIGNAL_FUNC (on_mainwindow_end_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "GTK Pattern Creation Lab");
  gtk_window_set_policy (GTK_WINDOW (mainwindow), FALSE, TRUE, TRUE);

  vbox = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (mainwindow), vbox);

  menubar2 = gtk_menu_bar_new ();
  gtk_widget_show (menubar2);
  gtk_box_pack_start (GTK_BOX (vbox), menubar2, FALSE, TRUE, 0);

  file = gtk_menu_item_new_with_label ("File");
  gtk_widget_show (file);
  gtk_container_add (GTK_CONTAINER (menubar2), file);

  menu19 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file), menu19);

  file_new = gtk_menu_item_new_with_label ("New");
  gtk_widget_show (file_new);
  gtk_container_add (GTK_CONTAINER (menu19), file_new);
  gtk_signal_connect (GTK_OBJECT (file_new), "activate",
                      GTK_SIGNAL_FUNC (on_file_new_activate),
                      NULL);
  gtk_widget_set_sensitive(file_new, FALSE);

  file_open = gtk_menu_item_new_with_label ("Open...");
  gtk_widget_show (file_open);
  gtk_container_add (GTK_CONTAINER (menu19), file_open);
  gtk_signal_connect (GTK_OBJECT (file_open), "activate",
                      GTK_SIGNAL_FUNC (on_file_open_activate),
                      NULL);
  gtk_widget_set_sensitive(file_open, FALSE);

  file_save = gtk_menu_item_new_with_label ("Save");
  gtk_widget_show (file_save);
  gtk_container_add (GTK_CONTAINER (menu19), file_save);
  gtk_signal_connect (GTK_OBJECT (file_save), "activate",
                      GTK_SIGNAL_FUNC (on_file_save_activate),
                      NULL);
  gtk_widget_set_sensitive(file_save, FALSE);

  file_save_as = gtk_menu_item_new_with_label ("Save As...");
  gtk_widget_show (file_save_as);
  gtk_container_add (GTK_CONTAINER (menu19), file_save_as);
  gtk_signal_connect (GTK_OBJECT (file_save_as), "activate",
                      GTK_SIGNAL_FUNC (on_file_save_as_activate),
                      NULL);
  gtk_widget_set_sensitive(file_save_as, FALSE);

  separator1 = gtk_menu_item_new ();
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (menu19), separator1);

  file_preferences = gtk_menu_item_new_with_label ("Preferences...");
  gtk_widget_show (file_preferences);
  gtk_container_add (GTK_CONTAINER (menu19), file_preferences);
  gtk_signal_connect (GTK_OBJECT (file_preferences), "activate",
                      GTK_SIGNAL_FUNC (on_file_preferences_activate),
                      NULL);

  separator2 = gtk_menu_item_new ();
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (menu19), separator2);

  file_quit = gtk_menu_item_new_with_label ("Quit");
  gtk_widget_show (file_quit);
  gtk_container_add (GTK_CONTAINER (menu19), file_quit);
  gtk_signal_connect (GTK_OBJECT (file_quit), "activate",
                      GTK_SIGNAL_FUNC (on_mainwindow_end_event),
                      NULL);

  pattern = gtk_menu_item_new_with_label ("Pattern");
  gtk_widget_show (pattern);
  gtk_container_add (GTK_CONTAINER (menubar2), pattern);

  menu20 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (pattern), menu20);

  create_patmenu(menu20);

  help = gtk_menu_item_new_with_label ("Help");
  gtk_widget_show (help);
  gtk_container_add (GTK_CONTAINER (menubar2), help);

  menu21 = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (help), menu21);

  help_about = gtk_menu_item_new_with_label ("About...");
  gtk_widget_show (help_about);
  gtk_container_add (GTK_CONTAINER (menu21), help_about);
  gtk_signal_connect (GTK_OBJECT (help_about), "activate",
                      GTK_SIGNAL_FUNC (on_help_about_activate),
                      NULL);

  generate_button = gtk_button_new_with_label ("Generate");
  gtk_object_set_data (GTK_OBJECT (mainwindow), "generate_button", generate_button);
  gtk_widget_show (generate_button);
  gtk_box_pack_start (GTK_BOX (vbox), generate_button, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (generate_button), "clicked",
                      GTK_SIGNAL_FUNC (on_generate_button_clicked),
                      NULL);
  gtk_widget_set_sensitive(generate_button, FALSE);

  interrupt_button = gtk_button_new_with_label ("Stop");
  gtk_widget_show (interrupt_button);
  gtk_box_pack_start (GTK_BOX (vbox), interrupt_button, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (interrupt_button, FALSE);
  gtk_signal_connect (GTK_OBJECT (interrupt_button), "clicked",
                      GTK_SIGNAL_FUNC (on_interrupt_button_clicked),
                      NULL);

  configurebox = gtk_vbox_new (FALSE, 3);
  gtk_object_set_data (GTK_OBJECT (mainwindow), "configurebox", configurebox);
  gtk_widget_show (configurebox);
  gtk_box_pack_start (GTK_BOX (vbox), configurebox, TRUE, TRUE, 0);

  return mainwindow;
}

GtkWidget*
create_colorselection_dialog (double* color)
{
  GtkWidget *colorselection_dialog;
  GtkWidget *color_selection_ok_button;
  GtkWidget *color_selection_cancel_button;
  GtkWidget *color_selection_help_button;

  colorselection_dialog = gtk_color_selection_dialog_new ("Select Color");
  gtk_container_border_width (GTK_CONTAINER (colorselection_dialog), 10);
  gtk_signal_connect (GTK_OBJECT (colorselection_dialog), "delete_event",
             GTK_SIGNAL_FUNC (on_color_selection_cancel_button_clicked),
             NULL);
  gtk_signal_connect (GTK_OBJECT (colorselection_dialog), "destroy_event",
             GTK_SIGNAL_FUNC (on_color_selection_cancel_button_clicked),
             NULL);
/*
  gtk_color_selection_set_opacity(GTK_COLOR_SELECTION(
             GTK_COLOR_SELECTION_DIALOG(colorselection_dialog)->colorsel),
             TRUE);
*/
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(
             GTK_COLOR_SELECTION_DIALOG(colorselection_dialog)->colorsel),
             color);

  color_selection_ok_button = GTK_COLOR_SELECTION_DIALOG (colorselection_dialog)->ok_button;
  gtk_widget_show (color_selection_ok_button);
  GTK_WIDGET_SET_FLAGS (color_selection_ok_button, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (color_selection_ok_button), "clicked",
             GTK_SIGNAL_FUNC (on_color_selection_ok_button_clicked),
             color);

  color_selection_cancel_button = GTK_COLOR_SELECTION_DIALOG (colorselection_dialog)->cancel_button;
  gtk_widget_show (color_selection_cancel_button);
  gtk_signal_connect (GTK_OBJECT (color_selection_cancel_button), "clicked",
             GTK_SIGNAL_FUNC (on_color_selection_cancel_button_clicked),
             NULL);

  color_selection_help_button = GTK_COLOR_SELECTION_DIALOG (colorselection_dialog)->help_button;
  gtk_widget_show (color_selection_help_button);

  return colorselection_dialog;
}

GtkWidget*
create_preferences_dialog ()
{
  GtkWidget *preferences_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *preferences_is_stretched_button;
  GtkWidget *preferences_show_progress_button;
  GtkWidget *hbox1;
  GtkObject *preferences_width_spinner_adj;
  GtkWidget *preferences_width_spinner;
  GtkWidget *preferences_label1;
  GtkWidget *hbox2;
  GtkObject *preferences_height_spinner_adj;
  GtkWidget *preferences_height_spinner;
  GtkWidget *preferences_label2;
  GtkWidget *hbox3;
  GtkObject *preferences_digits_spinner_adj;
  GtkWidget *preferences_digits_spinner;
  GtkWidget *preferences_label3;
  GtkWidget *preferences_action_area;
  GtkWidget *preferences_apply_button;
  GtkWidget *preferences_close_button;

  preferences_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (preferences_dialog), "Preferences");
  gtk_window_set_policy (GTK_WINDOW (preferences_dialog), FALSE, TRUE, TRUE);
  gtk_signal_connect(GTK_OBJECT(preferences_dialog), "delete_event",
                      GTK_SIGNAL_FUNC(on_preferences_close_button_clicked),
                      NULL);

  gtk_signal_connect(GTK_OBJECT(preferences_dialog), "destroy",
                      GTK_SIGNAL_FUNC(on_preferences_close_button_clicked),
                      NULL);

  dialog_vbox1 = GTK_DIALOG (preferences_dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (TRUE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  preferences_is_stretched_button = gtk_check_button_new_with_label ("Stretch");
  gtk_object_set_data (GTK_OBJECT (preferences_dialog), 
                       "preferences_is_stretched_button", 
                        preferences_is_stretched_button);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(preferences_is_stretched_button), gpreferences.is_stretched);
  gtk_widget_show (preferences_is_stretched_button);
  gtk_box_pack_start (GTK_BOX (vbox1), preferences_is_stretched_button, 
                      TRUE, TRUE, 0);

  preferences_show_progress_button = gtk_check_button_new_with_label ("Show Progress");
  gtk_object_set_data (GTK_OBJECT (preferences_dialog),
                      "preferences_show_progress_button",
                       preferences_show_progress_button);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(preferences_show_progress_button), gpreferences.show_progress);
  gtk_widget_show (preferences_show_progress_button);
  gtk_box_pack_start (GTK_BOX (vbox1), preferences_show_progress_button,
                      TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  preferences_width_spinner_adj = gtk_adjustment_new (gpreferences.width, 0, 1000, 1, 10, 10);
  preferences_width_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (preferences_width_spinner_adj), 1, 0);
  gtk_object_set_data (GTK_OBJECT (preferences_dialog), "preferences_width_spinner", preferences_width_spinner);
  gtk_widget_show (preferences_width_spinner);
  gtk_box_pack_start (GTK_BOX (hbox1), preferences_width_spinner, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (preferences_width_spinner), TRUE);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (preferences_width_spinner), TRUE);

  preferences_label1 = gtk_label_new ("Width");
  gtk_widget_show (preferences_label1);
  gtk_box_pack_start (GTK_BOX (hbox1), preferences_label1, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);

  preferences_height_spinner_adj = gtk_adjustment_new (gpreferences.height, 0, 1000, 1, 10, 10);
  preferences_height_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (preferences_height_spinner_adj), 1, 0);
  gtk_object_set_data (GTK_OBJECT (preferences_dialog), "preferences_height_spinner", preferences_height_spinner);
  gtk_widget_show (preferences_height_spinner);
  gtk_box_pack_start (GTK_BOX (hbox2), preferences_height_spinner, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (preferences_height_spinner), TRUE);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (preferences_height_spinner), TRUE);

  preferences_label2 = gtk_label_new ("Height");
  gtk_widget_show (preferences_label2);
  gtk_box_pack_start (GTK_BOX (hbox2), preferences_label2, TRUE, TRUE, 0);

  hbox3 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 0);

  preferences_digits_spinner_adj = gtk_adjustment_new (gpreferences.digits, 0, 5, 1, 10, 10);
  preferences_digits_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (preferences_digits_spinner_adj), 1, 0);
  gtk_object_set_data (GTK_OBJECT (preferences_dialog), "preferences_digits_spinner", preferences_digits_spinner);
  gtk_widget_show (preferences_digits_spinner);
  gtk_box_pack_start (GTK_BOX (hbox3), preferences_digits_spinner, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (preferences_digits_spinner), TRUE);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (preferences_digits_spinner), TRUE);

  preferences_label3 = gtk_label_new ("Spinner\nAccuracy");
  gtk_widget_show (preferences_label3);
  gtk_box_pack_start (GTK_BOX (hbox3), preferences_label3, TRUE, TRUE, 0);

  preferences_action_area = GTK_DIALOG (preferences_dialog)->action_area;
  gtk_widget_show (preferences_action_area);
  gtk_container_border_width (GTK_CONTAINER (preferences_action_area), 10);

  preferences_apply_button = gtk_button_new_with_label ("Apply");
  gtk_widget_show (preferences_apply_button);
  gtk_box_pack_start (GTK_BOX (preferences_action_area), preferences_apply_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (preferences_apply_button), "clicked",
                      GTK_SIGNAL_FUNC (on_preferences_apply_button_clicked),
                      NULL);

  preferences_close_button = gtk_button_new_with_label ("Close");
  gtk_widget_show (preferences_close_button);
  gtk_box_pack_start (GTK_BOX (preferences_action_area), preferences_close_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (preferences_close_button), "clicked",
                      GTK_SIGNAL_FUNC (on_preferences_close_button_clicked),
                      NULL);

  return preferences_dialog;
}

GtkWidget*
create_about_window ()
{
  GtkWidget *about_window;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *about_ok_button;

  about_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (about_window), "About GPCL");
  gtk_window_set_policy (GTK_WINDOW (about_window), FALSE, FALSE, TRUE);

  gtk_signal_connect(GTK_OBJECT(about_window), "delete_event",
                      GTK_SIGNAL_FUNC(on_about_ok_button_clicked),
                      NULL);

  gtk_signal_connect(GTK_OBJECT(about_window), "destroy",
                      GTK_SIGNAL_FUNC(on_about_ok_button_clicked),
                      NULL);

  vbox1 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (about_window), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (about_window), vbox1);

  label1 = gtk_label_new ("GPCL version 0.0.8\nCopyright (C) 1999 Peter Stuart");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, TRUE, TRUE, 0);
  gtk_misc_set_padding (GTK_MISC (label1), 5, 2);

  label2 = gtk_label_new ("GPCL comes with ABSOLUTELY\nNO WARRANTY. This is free\nsoftware and you are free to\ndistribute it under certain\nconditions. See accompanying\nCOPYING file.");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
  gtk_misc_set_padding (GTK_MISC (label2), 5, 2);

  about_ok_button = gtk_button_new_with_label ("OK");
  gtk_object_set_data (GTK_OBJECT (about_window), "about_ok_button", 
                      about_ok_button);
  gtk_widget_show (about_ok_button);
  gtk_box_pack_start (GTK_BOX (vbox1), about_ok_button, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (about_ok_button), 2);
  gtk_signal_connect (GTK_OBJECT (about_ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_about_ok_button_clicked),
                      NULL);

  return about_window;
}

GtkWidget* create_progress_window(void)
{
   GtkWidget *pbox;
   GtkWidget *pwindow;

   pwindow = gtk_dialog_new();
   gtk_signal_connect (GTK_OBJECT(pwindow), "destroy_event",
                       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                       &pwindow);
   gtk_signal_connect (GTK_OBJECT(pwindow), "delete_event",
                       GTK_SIGNAL_FUNC(on_interrupt_button_clicked),
                       NULL);

   gtk_window_set_title(GTK_WINDOW(pwindow), "Generating Image");
   gtk_container_border_width (GTK_CONTAINER(pwindow), 0);

   pbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(pwindow), 0);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pwindow)->vbox),
                      pbox, TRUE, TRUE, 0);
   gtk_widget_show(pbox);

   progress_label = gtk_label_new("Progress...");
   gtk_misc_set_alignment(GTK_MISC(progress_label), 0.0, 0.5);
   gtk_box_pack_start(GTK_BOX(pbox), progress_label, FALSE, TRUE, 0);
   gtk_widget_show(progress_label);

   progress_bar = gtk_progress_bar_new();
   gtk_widget_set_usize(progress_bar, 200, 20);
   gtk_box_pack_start(GTK_BOX(pbox), progress_bar, TRUE, TRUE, 0);
   gtk_widget_show(progress_bar);

   gtk_widget_show(pwindow);

   return pwindow;
}
