/***************************
slparser
Copyright (C) 1999  Peter Stuart

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************/


/**********MODULE DATASTACK***************
Filename: datastack.h
Author: ophiuchus42@hotmail.com
Description: This module implements stack operations for data of
  types string, float, and vector. This is used in the execution
  of RenderMan shading language shaders compiled with BMRT.
Modifications: 
  12/23/98   commented code
  12/30/98   added defines for free list
*************************/

#ifndef __DATASTACK__
#define __DATASTACK__

/* Number of elements to start in free list */
#define FREE_START 10
/* Number of elements to add to free list when all elements in use */
#define FREE_INCREMENT 5


/* initialize datastack */
void  datastack_init(PPDATA_STACK stack);

/* stack pop operations */
int  datastack_popv(PPDATA_STACK top, pvec3 v);
int  datastack_popf(PPDATA_STACK top, FLT* f);
int  datastack_pops(PPDATA_STACK top, char** s);
int  datastack_popc(PPDATA_STACK top, PCOLOR c);

/* stack push operations */
int  datastack_pushv(PPDATA_STACK top, pvec3 v);
int  datastack_pushf(PPDATA_STACK top, FLT* f);
int  datastack_pushs(PPDATA_STACK top, char** s);
int  datastack_pushc(PPDATA_STACK top, PCOLOR c);

/* free list of all elements */
void  datastack_delete(PPDATA_STACK top);

#endif
