/***************************
slparser
Copyright (C) 1999  Peter Stuart

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************/


/**********MODULE SLPARSER***************
Filename: slparser.h
Author: ophiuchus42@hotmail.com
Description: This module implements code to create shaders based
   on a token list. It is used to create the data structures for
	shaders as well as execute the shader itself. Currently the
	code is pretty unoptimized. There are many things in here that
	can be (and will be) changed.
Modifications: 
  12/24/98   commented code
*************************/

#ifndef __SLPARSER__
#define __SLPARSER__

/* initialize parser */
int slparser_init(PTOKEN_LIST list, PSHADER_DATA data);

/* go to code part of list */
void slparser_start(PPTOKEN_LIST current);

/* memory allocation function */
int slparser_malloc(int size, void** p);

/* array allocation function */
int slparser_calloc(int num, int size, void** p);

/* free memory of shader_data structure */
void slparser_delete(PSHADER_DATA data);

/* get value of input variable */
FLT* slparser_get_float(PSHADER_DATA data, char* var);
PCOLOR slparser_get_color(PSHADER_DATA data, char* var);
char** slparser_get_string(PSHADER_DATA data, char* var);
pvec3 slparser_get_vector(PSHADER_DATA data, char* var);

/* test token list for parse errors */
int slparser_test(PTOKEN_LIST list);

/* execute shader */
int slparser_execute(PTOKEN_LIST list, PPDATA_STACK stack, 
                     PGLOBAL_VARIABLES globals, PSHADER_DATA data);

#endif
