/***************************
slparser
Copyright (C) 1999  Peter Stuart

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************/


#include <stdlib.h>
#include "../include/matrix.h"
#include "../include/math3d.h"
#include "tokens.h"
#include "types.h"
#include "datastack.h"
#include "slparser.h"
#include "tokenlist.h"
#include "tokenizer.h"

void initializeglobals(PGLOBAL_VARIABLES globals);

void main(void)
{
   PTOKEN_LIST tokens = NULL;
   PTOKEN_LIST start = NULL;
   SHADER_DATA info;
   PDATA_STACK stack = NULL;
   GLOBAL_VARIABLES globals;
   int i, j;
/*   char filename[128];*/
/*   char* filename = "/home/ophiuchus/graphics/BMRT2.4/shaders/matte.slc";*/
  char* filename = "/home/ophiuchus/graphics/BMRT2.3.6/examples/wood2.slc";
   /*char* filename = "/home/ophiuchus/programming/projects/gpcl/shaders/brick.slc";*/

   initializeglobals(&globals);
   globals.Cs[0] = globals.Os[0] = 1.0;
   globals.Cs[1] = globals.Os[1] = 1.0;
   globals.Cs[2] = globals.Os[2] = 1.0;

   tokenlist_init(&tokens);
   datastack_init(&stack);

   if (!tokenizer_get_tokens(&tokens, filename))
      fprintf(stderr, "A problem occurred tokenizing %s\n", filename);

   if(slparser_init(tokens, &info))
   {
      printf("Name of shader is %s\n", info.shader_name);

      if (info.generator_name != NULL)
      {
         printf("Generator: %s\n", info.generator_name);
         printf("Version: %s\n", info.version_name);
      }

      for (i=0; i<info.float_params; i++)
      {
         printf("Parameter %s is %f\n", info.float_name[i], info.float_data[i]);
      }

      for (i=0; i<info.color_params; i++)
      {
         printf("Parameter %s is ", info.color_name[i]);
         vec3print(info.color_data[i]);
         printf("\n");
      }

      for (i=0; i<info.vector_params; i++)
      {
         printf("Parameter %s is ", info.vector_name[i]);
         vec3print(info.vector_data[i]);
         printf("\n");
      }

      for (i=0; i<info.string_params; i++)
         printf("Parameter %s is %s\n", info.string_name[i], info.string_data[i]);

      start = tokens;
      slparser_start(&start);
/*
      if (!slparser_test(start))
         printf("Parse Error!\n");
*/
      if (!slparser_execute(start, &stack, &globals, &info))
         printf("Error in execution\n");
   }

   printf("Resulting color is: ");
   vec3print(globals.Ci);
   printf("\n");

   printf("Resulting opacity is: ");
   vec3print(globals.Oi);
   printf("\n");
/*
   for (i=info.float_params; i<info.float_num; i++)
   {
      printf("Float variable %s is %f\n", info.float_name[i], info.float_data[i]);
   }

   for (i=info.color_params; i<info.color_num; i++)
   {
      printf("Color variable %s is ", info.color_name[i]);
      vec3print(info.color_data[i]);
      printf("\n");
   }

   for (i=info.vector_params; i<info.vector_num; i++)
   {
      printf("Vector variable %s is ", info.vector_name[i]);
      vec3print(info.vector_data[i]);
      printf("\n");
   }

   for (i=info.string_params; i<info.string_num; i++)
      printf("String variable %s is %s\n", info.string_name[i],
                                    info.string_data[i]);

   slparser_delete(&info);
   printf("info deleted\n");
*/
   tokenlist_delete(&tokens);
   printf("tokenlist deleted\n");
   datastack_delete(&stack);
   printf("datastack deleted\n");
}

void initializeglobals(PGLOBAL_VARIABLES globals)
{
   vec3init(globals->Cs);
   vec3init(globals->Os);
   vec3init(globals->P);
   vec3init(globals->dPdu);
   vec3init(globals->dPdv);
   vec3init(globals->N);
   globals->N[1] = 1.0;
   vec3init(globals->Ng);
   globals->Ng[1] = 1.0;
   globals->u = 0.0;
   globals->v = 0.0;
   globals->du = 0.0;
   globals->dv = 0.0;
   globals->s = 0.0;
   globals->t = 0.0;
   globals->dsdu = 0.1;
   globals->dsdv = 0.1;
   globals->dtdu = 0.1;
   globals->dtdv = 0.1;
   vec3init(globals->L);
   vec3init(globals->Cl);
   vec3init(globals->I);
   globals->I[1] = -1.0;
   vec3init(globals->Ci);
   globals->Ci[0] = globals->Oi[0] = 1.0;
   globals->Ci[1] = globals->Oi[1] = 1.0;
   globals->Ci[2] = globals->Oi[2] = 1.0;
   vec3init(globals->E);
   return;
}
