/***************************
slparser
Copyright (C) 1999  Peter Stuart

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************/


/**********TOKENS HEADER***************
Filename: tokens.h
Author: ophiuchus42@hotmail.com
Description: This header is used with all of the token modules.
   Modules dependent on this file include: tokenlist, tokenizer,
	datastack, and slparser. This header file should probably be split into
	three files when the tokenizer is generalized for use outside of
	RenderMan compiled shaders, as this file contains shading language
	specific code.
Modifications: 
  12/24/98   commented code
*************************/

#ifndef __SLTOKENS_H__
#define __SLTOKENS_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "slplib.h"
#include "../include/math3d.h"
#include "../include/matrix.h"

 /* #define __DEBUGTOKENLIST__ */
 /* #define __DEBUGTOKENIZER__ */
 /* #define __DEBUGSLPARSER__  */
 /* #define __DEBUGDATASTACK__ */

/* dmessage is used to print out debugging information. This is
   used in conjunction with the __DEBUG__ define. For ANSI C
	code dmessage should be printf. For GTK code use g_print.
   dmessage is defined to be nothing if __DEBUG__ is not defined.
*/

#define dmessage printf

 #define warning_message null_print
 /*#define warning_message printf */
#define error_message printf 

/* string compare macros */
#define streq(a, b)  (strcmp((a),(b)) == 0)
#define strneq(a, b) (strcmp((a),(b)) != 0)
#define strnneq(a, b, i) (strncmp((a),(b),(i)) != 0)

/* macros used to determine if a specified token is of a global type */
#define IS_GLOBAL_FLOAT(a) (((a)==U_GLOBAL_TOKEN) || ((a)==V_GLOBAL_TOKEN) \
                         || ((a)==DU_GLOBAL_TOKEN) || ((a)==DV_GLOBAL_TOKEN) \
                         || ((a)==S_GLOBAL_TOKEN) || ((a)==T_GLOBAL_TOKEN) \
                      || ((a)==DSDU_GLOBAL_TOKEN) || ((a)==DSDV_GLOBAL_TOKEN) \
                      || ((a)==DTDU_GLOBAL_TOKEN) || ((a)==DTDV_GLOBAL_TOKEN))
#define IS_GLOBAL_COLOR(a) (((a)==CS_GLOBAL_TOKEN) || ((a)==OS_GLOBAL_TOKEN) \
                         || ((a)==CL_GLOBAL_TOKEN) || ((a)==CI_GLOBAL_TOKEN) \
                         || ((a)==OI_GLOBAL_TOKEN))
#define IS_GLOBAL_VECTOR(a) (((a)==DPDU_GLOBAL_TOKEN) || ((a)==DPDV_GLOBAL_TOKEN) \
                         || ((a)==N_GLOBAL_TOKEN) || ((a)==NG_GLOBAL_TOKEN) \
                         || ((a)==L_GLOBAL_TOKEN) || ((a)==I_GLOBAL_TOKEN) \
                         || ((a)==P_GLOBAL_TOKEN) || ((a)==E_GLOBAL_TOKEN))

/* define TRUE and FALSE for use with ANSI C */
#ifndef FALSE
#define FALSE 0
#endif /* FALSE */

#ifndef TRUE
#define TRUE !FALSE
#endif /* TRUE */

/* MAXTOKENS is the number of tokens which should be reserved words */
#define MAXTOKENS		144

/* MAXCHARACTERS is the maximum number of characters which can be
   used as a word with the tokenizer */
#define MAXCHARACTERS		128

/************ RENDERMAN SHADING LANGUAGE TOKENS ************/

/* structure tokens */
#define CODE_TOKEN         0
#define INITCODE_TOKEN     1	
#define JUMP_TOKEN         2
#define LABEL_TOKEN        3	
#define LOCAL_TOKEN        4
#define PARAM_TOKEN        5
#define RETURN_AREA_TOKEN  6
#define RETURN_TOKEN       7
#define SURFACE_TOKEN      8
#define VERSION_TOKEN      9

/* data type tokens */
#define COLOR_TOKEN        20
#define FLOAT_TOKEN        21
#define NORMAL_TOKEN       22
#define POINT_TOKEN        23
#define STRING_TOKEN       24
#define VECTOR_TOKEN       25

/* stack operation tokens */
#define POPC_TOKEN         40 /* pop color */
#define POPF_TOKEN         42 /* pop float */
#define POPV_TOKEN         48 /* pop vector */
#define PUSHC_TOKEN        41 /* push color */     
#define PUSHF_TOKEN        43 /* push float */
#define PUSHIF_TOKEN       44 /* push float input */
#define PUSHIS_TOKEN       45 /* push string input */
#define PUSHIV_TOKEN       46 /* push vector input */
#define PUSHS_TOKEN        47 /* push string */
#define PUSHV_TOKEN        49 /* push vector */

/* operator tokens */
#define ADDC_IN_TOKEN      60 /* add input color */
#define ADDCC_TOKEN        61 /* add two colors */
#define ADDCF_TOKEN        62 /* add color to float */
#define ADDF_IN_TOKEN      63 /* add float to input float */
#define ADDFF_TOKEN        64 /* add two floats */
#define ADDFF_ININ_TOKEN   65 /* add two input floats */
#define ADDV_IN_TOKEN      66 /* add vector to input vector */
#define ADDVV_TOKEN        67 /* add two vectors */
#define CPCC_IN_TOKEN      68 /* assign input color to input color */
#define CPFF_IN_TOKEN      69 /* assign input float to input float */
#define CROSS_TOKEN        70 /* cross product */
#define DIVF_IN_TOKEN      71 /* divide float by input float variable */
#define DIVFF_TOKEN        72 /* divide two floats */
#define DIVFF_ININ_TOKEN   73 /* divide two input floats */
#define DIVVF_TOKEN        74 /* divide vector by float */
#define FTOC_TOKEN         75 /* convert float to color */
#define MULC_IN_TOKEN      76 /* multiply top by input color */
#define MULCC_TOKEN        77 /* multiply two colors */
#define MULCC_ININ_TOKEN   78 
#define MULCF_TOKEN        79 /* multiply color by float */
#define MULCF_ININ_TOKEN   97 /* multiply input color by input float */
#define MULF_IN_TOKEN      80 /* multiply top by input float */
#define MULFC_TOKEN        81 /* multiply float by color */
#define MULFC_IN_TOKEN     82
#define MULFF_TOKEN        83 /* multiply two floats */
#define MULFF_ININ_TOKEN   84 /* multiply two input floats */
#define MULFV_IN_TOKEN     85 /* multiply vector by input float */
#define MULFV_TOKEN        86 /* multiply vector by float */
#define MULVF_TOKEN        87 /* multiply vector by float */
#define NEGF_TOKEN         88 /* negative float */
#define NEGV_TOKEN         89 /* negative vector */
#define SUBCC_TOKEN        90 /* subtract two colors */
#define SUBCF_TOKEN        91 /* subtract float from color */
#define SUBF_IN_TOKEN      92 /* subtract float by input float */
#define SUBFF_TOKEN        93 /* subtract two floats */
#define SUBFF_ININ_TOKEN   94 /* subtract two input floats */
#define SUBVV_TOKEN        95 /* subtract two vectors */
#define VDOT_TOKEN         96 /* dot product of two vectors */

/* relational tokens */
#define AND_TOKEN          100 /* boolean AND */
#define FEQF_TOKEN         101 /* floats equal */
#define FNEF_TOKEN         102 /* float not equal */
#define GE_TOKEN           103 /* greater than or equal */
#define GT_TOKEN           104 /* greater than */
#define IFZ_TOKEN          105 /* if argument goto label z*/
#define LT_TOKEN           106 /* less than */
#define OR_TOKEN           107 /* boolean OR */
#define SNE_TOKEN          108 /* strings not equal */

/* geometric functions tokens */
#define AREA_TOKEN         120 /* surface element area */
#define CALCULATENORMAL_TOKEN 121 /* normal to surface at point */
#define COMP_TOKEN         124 /* get component of color */
#define FACEFORWARD_TOKEN  127 /* faceforward vector */
#define LENGTH_TOKEN       128 /* length of vector */
#define NORMALIZE_TOKEN    129 /* normalize vector */
#define NTRANSFORMS_TOKEN  130
#define TRANSFORMS_TOKEN   132 /* transform to coordinate system */
#define VTRANSFORMS_TOKEN  131
#define XCOMP_TOKEN        133 /* x component of vector */
#define YCOMP_TOKEN        134 /* y component of vector */
#define ZCOMP_TOKEN        135 /* z component of vector */

/* mathematical functions tokens */
#define ABS_TOKEN          150 /* absolute value */
#define EXP_TOKEN          156 /* exponential */
#define FCLAMP_TOKEN       157 /* clamp */
#define FLOOR_TOKEN        158 /* floor */
#define FMAX_TOKEN         159 /* maximum */
#define FMIN_TOKEN         160 /* minimum */
#define FRESNEL4_TOKEN     161
#define FTEXTURE_TOKEN     162
#define MOD_TOKEN          163 /* mod */
#define POW_TOKEN          169 /* power */
#define RANDOM_TOKEN       170
#define SMOOTHSTEP_TOKEN   171 /* smoothstep */
#define SQRT_TOKEN         172
#define TRACE_TOKEN        173

/* illumination tokens */ 
#define AMBIENT_TOKEN      180 /* ambient light */
#define CENVIRONMENT_TOKEN 181
#define DIFFUSE_TOKEN      182 /* diffuse light */
#define ILLUMINANCE3_TOKEN 183
#define ILLUMINANCE_START_TOKEN 184
#define LIGHTSOURCE_TOKEN  185
#define REFLECT_TOKEN      186
#define SPECULAR_TOKEN     187 /* specular highlight */

/* miscellaneous tokens */
#define CMIX_TOKEN         122 /* mix two colors */
#define COLORNOISEV_TOKEN  123 /* color noise */
#define CSET_TOKEN         125 /* set each component of color */
#define CTEXTURE_TOKEN     126 /* UNKNOWN */
#define CELLNOISEF_TOKEN   151
#define CELLNOISEFF_TOKEN  152
#define DERIVV_TOKEN       153
#define DUF_TOKEN          155
#define DVF_TOKEN          154
#define NOISEF_TOKEN       164 
#define NOISEFF_TOKEN      165
#define NOISEV_TOKEN       166
#define POINTNOISEFF_TOKEN 167
#define POINTNOISEV_TOKEN  168

/* global variables tokens */
#define CS_GLOBAL_TOKEN    190 /* surface color (input) */
#define OS_GLOBAL_TOKEN    191 /* surface opacity (input) */
#define P_GLOBAL_TOKEN     192 /* surface position */
#define DPDU_GLOBAL_TOKEN  193 /* change in position with u */
#define DPDV_GLOBAL_TOKEN  194 /* change in position with v */
#define N_GLOBAL_TOKEN     195 /* surface shading normal */
#define NG_GLOBAL_TOKEN    196 /* surface geometric normal */
#define U_GLOBAL_TOKEN     197 /* surface u parameter */
#define V_GLOBAL_TOKEN     198 /* surface v parameter */
#define DU_GLOBAL_TOKEN    199 /* change in u across element */
#define DV_GLOBAL_TOKEN    200 /* change in v across element */
#define S_GLOBAL_TOKEN     201 /* texture s coordinate */
#define T_GLOBAL_TOKEN     202 /* texture t coordinate */
#define DSDU_GLOBAL_TOKEN  203 /* change of s with respect to u */
#define DSDV_GLOBAL_TOKEN  204 /* change of s with respect to v */
#define DTDU_GLOBAL_TOKEN  205 /* change of t with respect to u */
#define DTDV_GLOBAL_TOKEN  206 /* change of t with respect to v */
#define L_GLOBAL_TOKEN     207 /* direction to light */
#define CL_GLOBAL_TOKEN    208 /* light color */
#define I_GLOBAL_TOKEN     209 /* direction of ray */
#define CI_GLOBAL_TOKEN    210 /* color from surface (output) */
#define OI_GLOBAL_TOKEN    211 /* opacity from surface (output) */
#define E_GLOBAL_TOKEN     212 /* position of camera */

/* unknown tokens */
#define ALPHA_TOKEN        215 /* string */
#define EOF_TOKEN          216 /* end of file */
#define NUMERIC_TOKEN      217 /* number */

#endif
